/*
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
| File Name			  : MicroBolt_AtoD.c																					
| Author			    : Micromint, Inc.						
| Copyright       : Copyright  2005, Micromint, Inc.																	
| Creation Date		: 9/30/05																							
| Version			    : 1.00																																												
| Spaces per tab	:	2																									
| Description		  : Main C file																																									 
| Revision			  :	Initial																																																						
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
*/
 
/*
|----------------------------------------------------
|     Includes
|----------------------------------------------------
*/

#include <ARM/philips/lpc210x.h>
#include <arm_macros.h>
#include "MicroBolt_AtoD.h"
#include "SerialDebugger.h"

/*
|------------------------------------------------------------------------------------------
|     Function        :   main
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Main function for system
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

void main(void)
  {
  unsigned int Delay = 0;
  unsigned char Count = 0;
  unsigned short AtoDvalueChan0, AtoDvalueChan1;

  __DISABLE_INTERRUPT();                        // Disable all interrupts

/*
|-------------------------
| Config MAM 
|-------------------------
*/
  MAM_CR = 0x00;                                // Turn MAM off (default)
  MAM_TIM = 0x04;                               // Set flash timing to 4 clock cycles

  MAM_CR = 0x02;                                // Fully enable the Memory Accleration Module

/*
|-------------------------
| Configure VIC 
|-------------------------
*/
  VICVectAddr0 = (unsigned)pll_isr;             // Assign the PLL lock ISR vector address
  VICVectCntl0 = INTERRUPT_CHANNEL_FOR_PLL;     // Assign the VIC address to the actual interrupt
  VICIntEnable = INTERRUPT_ENABLE_FOR_PLL;      // Enable the interrupt  

/*
|-------------------------
| Config PLL and CCLK 
|-------------------------
*/
  SCB_PLLCFG |= 0x23;				 	   		            // Set to 59 MHz (0x03 is multiply value of 4)
  SCB_PLLCON |= 0x01;                           // Enable the PLL 
  SCB_PLLFEED = 0xAA;					   	  	          // Shadow register copy to enable changes
  SCB_PLLFEED = 0x55;                           // in PLLCON and PLLCFG

/*
|-------------------------
| Config PCLK 
|-------------------------
*/
  SCB_VPBDIV = 0;                               // Peripheral clock is 1/4th Processor clock which equals 14.7456 MHz

/*
|-------------------------
| Config GPIO 
|-------------------------
*/
  PCB_PINSEL0=0x00005505;                       // Setup with ICCARM App builder - MicroBolt_AtoD.bcf (UART0 and SPI pins)   
  PCB_PINSEL1=0x55400000;                       //                                                    (Secondary JTAG pins) 

  GPIO_IODIR |= MICROBOLT_LED;                  // Setup MicroBolt LED as output
  GPIO_IODIR |= A_TO_D_CHIP_SELECT;             // Setup A to D chip select - P0.16 

  GPIO_IOCLR=0xffffffff;                        // Clear all pins to start with

	GPIO_IOSET = A_TO_D_CHIP_SELECT;              // Chip select high to start with - Disabled

/*
|-------------------------
| Config SPI (Master) 
|-------------------------
*/
 	SPI_SPCR=0x00000020;													// Setup edges
 	SPI_SPCCR=0x00000052;													// Clock frequency 170 KHz	

/*
|-------------------------
| Config UART-O 
|-------------------------
*/
  UART0_IER = 0x00000001;	 	                   // Receive interrupts
  UART0_FCR = 0x00000001;	 	                   // Enable the fifos
  UART0_LCR = 0x00000083;	 	                   // Enable the divisor
  UART0_DLM = 0;				                       // Divisor latch MSB (for baud rates < 4800) 
  UART0_DLL = BAUD_RATE_115200;			           // Divisor latch LSB 
  UART0_LCR = 0x00000003;  	                   // Close it, then UART works with divisor

  __ENABLE_INTERRUPT();                        // Enable all interrupts

/*
|----------------------------------------------------
|     Start of application
|----------------------------------------------------
*/

  while(1)                                                      // Do this forever
    {
    AtoDvalueChan0 = ReadAtoD(CHANNEL_0);                       // Read A to D channel 0
    AtoDvalueChan1 = ReadAtoD(CHANNEL_1);                       // Read A to D channel 1
                                                  
    SerialDebug(1,"MCP3202 Chan-0 value: %d", AtoDvalueChan0);  // Output string to MicroBolt Serial Debugger Debug window 1
    SerialDebug(2,"MCP3202 Chan-1 value: %d", AtoDvalueChan1);  // Output string to MicroBolt Serial Debugger Debug window 2  
       
    for (Delay = 0; Delay < 870000; Delay++);                   // Delay for 100 mS

    Count++;                                                    // Now simply blink the serial debugger LED
    if (Count == 10)
      {
      SerialDebugLed(LED_ON);
      GPIO_IOSET = MICROBOLT_LED;
      }
    else
    if (Count == 20)
      {
      Count = 0;
      SerialDebugLed(LED_OFF);
      GPIO_IOCLR = MICROBOLT_LED;
      }
    }
}

/*
|------------------------------------------------------------------------------------------
|     Function        :   ReadAtoD
|     Inputs          :   None
|     Outputs         :   Analog value from the A to D
|     Purpose         :   Read A to D values from MCP3202
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

unsigned short ReadAtoD(unsigned char AtoDchannelNumber)
  {
	unsigned char AnalogToDigitalMsbValue;
	unsigned char AnalogToDigitalLsbValue;
	unsigned short AnalogToDigital12bitValue;

	GPIO_IOCLR = A_TO_D_CHIP_SELECT;                                  // Chip select low - Enable
	Delay10uS();																											// Setup time

	SPI_SPDR = A_TO_D_START_SINGLE;                                   // Send out first byte to A to D
	while(!(SPI_SPSR & SPIF));                                        // Wait for completion

  if (AtoDchannelNumber == CHANNEL_0)                               // Send out 2nd byte, data depends on channel number
      {
	    SPI_SPDR = A_TO_D_CHANNEL_0;
      }
  else
      {
	    SPI_SPDR = A_TO_D_CHANNEL_1;
      }    
	while(!(SPI_SPSR & SPIF));                                        // Wait for completion

	AnalogToDigitalMsbValue =  SPI_SPDR;                              // Got the first desired byte (MSB) of A to D data, now store it

	SPI_SPDR = A_TO_D_NULL_WAIT_FOR_DATA;                             // Send out 3rd byte of data so as to get the second A to D byte
	while(!(SPI_SPSR & SPIF));                                        // Wait for completion

	AnalogToDigitalLsbValue =  SPI_SPDR;                              // Got the second byte (LSB) of A to D data, now store it

	Delay10uS();																										  // Hold time
	GPIO_IOSET = A_TO_D_CHIP_SELECT;                                  // Chip select high - Disable
                                                                
                                                                    // Now create 12 bit A to D value...

	AnalogToDigitalMsbValue = AnalogToDigitalMsbValue & 0x0F;																						// Drop top 4 bits from MSB for 12 bit value
	AnalogToDigital12bitValue = (unsigned short)AnalogToDigitalMsbValue << 8;                           // Store MSB by shifting up 8
	AnalogToDigital12bitValue = AnalogToDigital12bitValue | (unsigned short) AnalogToDigitalLsbValue;		// OR in the LSB to get the final value

	return(AnalogToDigital12bitValue);                                // 12 bit A to D value read from MCP3202, return it
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   Delay10uS
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Delay for 10 uS when PLLCFG is set to 0x23 (58.98 MHz) 
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

void Delay10uS(void)   
  {
	volatile unsigned int Delay;

  for (Delay = 0; Delay < 35; Delay++);          // Delay for 10 uS 
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   pll_isr
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Once PLL has locked, connect it and use for system clock
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

#pragma interrupt_handler pll_isr
void pll_isr(void)
  {
  SCB_PLLCON |= 0x02;                           // Connect the PLL 
  SCB_PLLFEED = 0xAA;					   	  	          // Shadow register copy to enable changes
  SCB_PLLFEED = 0x55;                           // in PLLCON and PLLCFG
  VICIntEnClear = PLL_CLR;                      // Disable the PLL interrupt, not needed anymore
  VICVectAddr = VIC_ACK;                        // Acknowledge Interrupt
  }
