/*
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
| File Name			  : SerialDebugger.c																					
| Author			    : Micromint, Inc.						
| Copyright       : Copyright  2005, Micromint, Inc.																	
| Creation Date		: 9/30/05																							
| Version			    : 1.00																																												
| Spaces per tab	:	2																									
| Description		  : Serial Debugger C file																																									 
| Revision			  :	Initial																																																						
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
*/

/*
|----------------------------------------------------
|     Includes
|----------------------------------------------------
*/

#include <ARM/philips/lpc210x.h>
#include <arm_macros.h>
#include <STDIO.h>
#include <STDARG.h>

#include "SerialDebugger.h"

/*
|------------------------------------------------------------------------------------------
|     Function        :   main
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Main function for system
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

extern int _textmode;		                        // This is defined in the ICCARM library 
		
/*
|------------------------------------------------------------------------------------------
|     Function        :   SerialDebug
|     Inputs          :   Printf for serial debugger
|     Outputs         :   None
|     Purpose         :   Transmit data to the MicroBolt Serial Debugger
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

void SerialDebug(char DebugBoxNum, char *fmt,...)
    {
      va_list va;
      va_start(va, fmt);          
      printf("%d", DebugBoxNum);
      vprintf(fmt,va);
      printf("\n");
    }

/*
|------------------------------------------------------------------------------------------
|     Function        :   SerialDebugLed
|     Inputs          :   LED for serial debugger
|     Outputs         :   None
|     Purpose         :   Transmit data to the MicroBolt Serial Debugger
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

void SerialDebugLed(char LedOnOrOff)
    {
      if (LedOnOrOff == LED_ON)
        {
        printf("L1\n");
        }
      else
      if (LedOnOrOff == LED_OFF)
        {
        printf("L0\n");
        }
    }

/*
|------------------------------------------------------------------------------------------
|     Function        :   putchar
|     Inputs          :   Byte for transmission
|     Outputs         :   None
|     Purpose         :   Transmit a byte via UART0 to enable printf
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

int putchar(char SerialByte)
  {
  _textmode = 1;                                // Turn carriage return line feed on for printf
  if (_textmode == 1 && SerialByte == '\n')
      {
      putchar('\r');
      }
  while (!(UART0_LSR & 0x20)); 
  UART0_THR = SerialByte;
  return SerialByte;
  }


